"""Constants used in the videodb package."""
from typing import Union
from dataclasses import dataclass

VIDEO_DB_API: str = "https://api.videodb.io"


class MediaType:
    video = "video"
    audio = "audio"
    image = "image"


class SearchType:
    semantic = "semantic"
    keyword = "keyword"
    scene = "scene"
    llm = "llm"


class IndexType:
    spoken_word = "spoken_word"
    scene = "scene"


class SceneExtractionType:
    shot_based = "shot"
    time_based = "time"


class Workflows:
    add_subtitles = "add_subtitles"


class SemanticSearchDefaultValues:
    result_threshold = 5
    score_threshold = 0.2


class Segmenter:
    time = "time"
    word = "word"
    sentence = "sentence"


class ApiPath:
    collection = "collection"
    upload = "upload"
    video = "video"
    audio = "audio"
    image = "image"
    stream = "stream"
    thumbnail = "thumbnail"
    thumbnails = "thumbnails"
    upload_url = "upload_url"
    transcription = "transcription"
    index = "index"
    search = "search"
    compile = "compile"
    workflow = "workflow"
    timeline = "timeline"
    delete = "delete"
    billing = "billing"
    usage = "usage"
    invoices = "invoices"
    scenes = "scenes"
    scene = "scene"
    frame = "frame"
    describe = "describe"
    storage = "storage"
    download = "download"
    title = "title"


class Status:
    processing = "processing"
    in_progress = "in progress"


class HttpClientDefaultValues:
    max_retries = 1
    timeout = 30
    backoff_factor = 0.1
    status_forcelist = [502, 503, 504]


class MaxSupported:
    fade_duration = 5


class SubtitleBorderStyle:
    no_border = 1
    opaque_box = 3
    outline = 4


class SubtitleAlignment:
    bottom_left = 1
    bottom_center = 2
    bottom_right = 3
    middle_left = 9
    middle_center = 10
    middle_right = 11
    top_left = 5
    top_center = 6
    top_right = 7


@dataclass
class SubtitleStyle:
    font_name: str = "Arial"
    font_size: float = 18
    primary_colour: str = "&H00FFFFFF"  # white
    secondary_colour: str = "&H000000FF"  # blue
    outline_colour: str = "&H00000000"  # black
    back_colour: str = "&H00000000"  # black
    bold: bool = False
    italic: bool = False
    underline: bool = False
    strike_out: bool = False
    scale_x: float = 1.0
    scale_y: float = 1.0
    spacing: float = 0
    angle: float = 0
    border_style: int = SubtitleBorderStyle.outline
    outline: float = 1.0
    shadow: float = 0.0
    alignment: int = SubtitleAlignment.bottom_center
    margin_l: int = 10
    margin_r: int = 10
    margin_v: int = 10


@dataclass
class TextStyle:
    fontsize: int = 24
    fontcolor: str = "black"
    fontcolor_expr: str = ""
    alpha: float = 1.0
    font: str = "Sans"
    box: bool = True
    boxcolor: str = "white"
    boxborderw: str = "10"
    boxw: int = 0
    boxh: int = 0
    line_spacing: int = 0
    text_align: str = "T"
    y_align: str = "text"
    borderw: int = 0
    bordercolor: str = "black"
    expansion: str = "normal"
    basetime: int = 0
    fix_bounds: bool = False
    text_shaping: bool = True
    shadowcolor: str = "black"
    shadowx: int = 0
    shadowy: int = 0
    tabsize: int = 4
    x: Union[str, int] = "(main_w-text_w)/2"
    y: Union[str, int] = "(main_h-text_h)/2"



import webbrowser as web
PLAYER_URL: str = "https://console.videodb.io/player"


def play_stream(url: str):
    """Play a stream url in the browser/ notebook

    :param str url: The url of the stream
    :return: The player url if the stream is opened in the browser or the iframe if the stream is opened in the notebook
    """
    player = f"{PLAYER_URL}?url={url}"
    opend = web.open(player)
    if not opend:
        try:
            from IPython.display import IFrame

            player_width = 800
            player_height = 400
            return IFrame(player, player_width, player_height)
        except ImportError:
            return player
    return player



from videodb.exceptions import (
    VideodbError,
    AuthenticationError,
    InvalidRequestError,
    SearchError,
)


def test_videodb_error():
    try:
        raise VideodbError("An error occurred", cause="Something")

    except VideodbError as e:
        assert str(e) == "An error occurred caused by Something"
        assert e.cause == "Something"


def test_authentication_error():
    try:
        raise AuthenticationError(
            "An error occurred with authentication", response="Something"
        )

    except AuthenticationError as e:
        print(e)
        assert str(e) == "An error occurred with authentication "
        assert e.response == "Something"


def test_invalid_request_error():
    try:
        raise InvalidRequestError(
            "An error occurred with request", response="Something"
        )

    except InvalidRequestError as e:
        assert str(e) == "An error occurred with request "
        assert e.response == "Something"


def test_search_error():
    try:
        raise SearchError("An error occurred with search")

    except SearchError as e:
        assert str(e) == "An error occurred with search "



"""Http client Module."""

import logging
import requests
import backoff

from tqdm import tqdm
from typing import (
    Callable,
    Optional,
)
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

from videodb._constants import (
    HttpClientDefaultValues,
    Status,
)
from videodb.exceptions import (
    AuthenticationError,
    InvalidRequestError,
    RequestTimeoutError,
)

logger = logging.getLogger(__name__)


class HttpClient:
    """Http client for making requests"""

    def __init__(
        self,
        api_key: str,
        base_url: str,
        version: str,
        max_retries: Optional[int] = HttpClientDefaultValues.max_retries,
    ) -> None:
        """Create a new http client instance

        :param str api_key: The api key to use for authentication
        :param str base_url: The base url to use for the api
        :param int max_retries: (optional) The maximum number of retries to make for a request
        """
        self.session = requests.Session()

        retries = Retry(
            total=max_retries,
            backoff_factor=HttpClientDefaultValues.backoff_factor,
            status_forcelist=HttpClientDefaultValues.status_forcelist,
        )
        adapter = HTTPAdapter(max_retries=retries)
        self.session.mount("http://", adapter)
        self.session.mount("https://", adapter)
        self.version = version
        self.session.headers.update(
            {
                "x-access-token": api_key,
                "x-videodb-client": f"videodb-python/{self.version}",
                "Content-Type": "application/json",
            }
        )
        self.base_url = base_url
        self.show_progress = False
        self.progress_bar = None
        logger.debug(f"Initialized http client with base url: {self.base_url}")

    def _make_request(
        self,
        method: Callable[..., requests.Response],
        path: str,
        base_url: Optional[str] = None,
        headers: Optional[dict] = None,
        **kwargs,
    ):
        """Make a request to the api

        :param Callable method: The method to use for the request
        :param str path: The path to make the request to
        :param str base_url: (optional) The base url to use for the request
        :param dict headers: (optional) The headers to use for the request
        :param kwargs: The keyword arguments to pass to the request method
        :return: json response from the request
        """
        try:
            url = f"{base_url or self.base_url}/{path}"
            timeout = kwargs.pop("timeout", HttpClientDefaultValues.timeout)
            request_headers = {**self.session.headers, **(headers or {})}
            response = method(url, headers=request_headers, timeout=timeout, **kwargs)
            response.raise_for_status()
            return self._parse_response(response)

        except requests.exceptions.RequestException as e:
            self._handle_request_error(e)

    def _handle_request_error(self, e: requests.exceptions.RequestException) -> None:
        """Handle request errors"""
        self.show_progress = False
        if isinstance(e, requests.exceptions.HTTPError):
            try:
                error_message = e.response.json().get("message", "Unknown error")
            except ValueError:
                error_message = e.response.text

            if e.response.status_code == 401:
                raise AuthenticationError(
                    f"Error: {error_message}", e.response
                ) from None
            else:
                raise InvalidRequestError(
                    f"Invalid request: {error_message}", e.response
                ) from None

        elif isinstance(e, requests.exceptions.RetryError):
            raise InvalidRequestError(
                "Invalid request: Max retries exceeded", e.response
            ) from None

        elif isinstance(e, requests.exceptions.Timeout):
            raise RequestTimeoutError(
                "Timeout error: Request timed out", e.response
            ) from None

        elif isinstance(e, requests.exceptions.ConnectionError):
            raise InvalidRequestError(
                "Invalid request: Connection error", e.response
            ) from None

        else:
            raise InvalidRequestError(
                f"Invalid request: {str(e)}", e.response
            ) from None

    @backoff.on_exception(
        backoff.constant, Exception, max_time=500, interval=5, logger=None, jitter=None
    )
    def _get_output(self, url: str):
        """Get the output from an async request"""
        response_json = self.session.get(url).json()
        if (
            response_json.get("status") == Status.in_progress
            or response_json.get("status") == Status.processing
        ):
            percentage = response_json.get("data", {}).get("percentage")
            if percentage and self.show_progress and self.progress_bar:
                self.progress_bar.n = int(percentage)
                self.progress_bar.update(0)

            logger.debug("Waiting for processing to complete")
            raise Exception("Stuck on processing status") from None
        if self.show_progress and self.progress_bar:
            self.progress_bar.n = 100
            self.progress_bar.update(0)
            self.progress_bar.close()
            self.progress_bar = None
            self.show_progress = False
        return response_json.get("response") or response_json

    def _parse_response(self, response: requests.Response):
        """Parse the response from the api"""
        try:
            response_json = response.json()
            if (
                response_json.get("status") == Status.processing
                and response_json.get("request_type", "sync") == "async"
            ):
                return None
            elif (
                response_json.get("status") == Status.processing
                and response_json.get("request_type", "sync") == "sync"
            ):
                if self.show_progress:
                    self.progress_bar = tqdm(
                        total=100,
                        position=0,
                        leave=True,
                        bar_format="{l_bar}{bar:100}{r_bar}{bar:-100b}",
                    )
                response_json = self._get_output(
                    response_json.get("data", {}).get("output_url")
                )
                if response_json.get("success"):
                    return response_json.get("data")
                else:
                    raise InvalidRequestError(
                        f"Invalid request: {response_json.get('message')}", response
                    ) from None

            elif response_json.get("success"):
                return response_json.get("data")

            else:
                raise InvalidRequestError(
                    f"Invalid request: {response_json.get('message')}", response
                ) from None

        except ValueError:
            raise InvalidRequestError(
                f"Invalid request: {response.text}", response
            ) from None

    def get(
        self, path: str, show_progress: Optional[bool] = False, **kwargs
    ) -> requests.Response:
        """Make a get request"""
        self.show_progress = show_progress
        return self._make_request(method=self.session.get, path=path, **kwargs)

    def post(
        self, path: str, data=None, show_progress: Optional[bool] = False, **kwargs
    ) -> requests.Response:
        """Make a post request"""
        self.show_progress = show_progress
        return self._make_request(self.session.post, path, json=data, **kwargs)

    def put(self, path: str, data=None, **kwargs) -> requests.Response:
        """Make a put request"""
        return self._make_request(self.session.put, path, json=data, **kwargs)

    def delete(self, path: str, **kwargs) -> requests.Response:
        """Make a delete request"""
        return self._make_request(self.session.delete, path, **kwargs)

    def patch(self, path: str, data=None, **kwargs) -> requests.Response:
        """Make a patch request"""
        return self._make_request(self.session.patch, path, json=data, **kwargs)



"""videodb.exceptions

This module contains the set of Videodb's exceptions.
"""


class VideodbError(Exception):
    """
    Base class for all videodb exceptions.
    """

    def __init__(self, message: str = "An error occurred", cause=None):
        super(VideodbError, self).__init__(message)
        self.cause = cause

    def __str__(self):
        return f"{super(VideodbError, self).__str__()} {'caused by ' + str(self.cause) if self.cause else ''}"


class AuthenticationError(VideodbError):
    """
    Raised when authentication is required or failed.
    """

    def __init__(self, message, response=None):
        super(AuthenticationError, self).__init__(message)
        self.response = response


class InvalidRequestError(VideodbError):
    """
    Raised when a request is invalid.
    """

    def __init__(self, message, response=None):
        super(InvalidRequestError, self).__init__(message)
        self.response = response


class RequestTimeoutError(VideodbError):
    """
    Raised when a request times out.
    """

    def __init__(self, message, response=None):
        super(RequestTimeoutError, self).__init__(message)
        self.response = response


class SearchError(VideodbError):
    """
    Raised when a search is invalid.
    """

    def __init__(self, message):
        super(SearchError, self).__init__(message)



from videodb._constants import (
    ApiPath,
)


class Image:
    def __init__(self, _connection, id: str, collection_id: str, **kwargs) -> None:
        self._connection = _connection
        self.id = id
        self.collection_id = collection_id
        self.name = kwargs.get("name", None)
        self.url = kwargs.get("url", None)

    def __repr__(self) -> str:
        return (
            f"Image("
            f"id={self.id}, "
            f"collection_id={self.collection_id}, "
            f"name={self.name}, "
            f"url={self.url})"
        )

    def delete(self) -> None:
        self._connection.delete(f"{ApiPath.image}/{self.id}")


class Frame(Image):
    def __init__(
        self,
        _connection,
        id: str,
        video_id: str,
        scene_id: str,
        url: str,
        frame_time: float,
        description: str,
    ):
        super().__init__(_connection=_connection, id=id, collection_id=None, url=url)
        self.scene_id = scene_id
        self.video_id = video_id
        self.frame_time = frame_time
        self.description = description

    def __repr__(self) -> str:
        return (
            f"Frame("
            f"id={self.id}, "
            f"video_id={self.video_id}, "
            f"scene_id={self.scene_id}, "
            f"url={self.url}, "
            f"frame_time={self.frame_time}, "
            f"description={self.description})"
        )

    def to_json(self):
        return {
            "id": self.id,
            "video_id": self.video_id,
            "scene_id": self.scene_id,
            "url": self.url,
            "frame_time": self.frame_time,
            "description": self.description,
        }

    def describe(self, prompt: str = None, model_name=None):
        description_data = self._connection.post(
            path=f"{ApiPath.video}/{self.video_id}/{ApiPath.frame}/{self.id}/{ApiPath.describe}",
            data={"prompt": prompt, "model_name": model_name},
        )
        self.description = description_data.get("description", None)
        return self.description



import requests

from typing import Optional
from requests import HTTPError


from videodb._constants import (
    ApiPath,
)

from videodb.exceptions import (
    VideodbError,
)


def upload(
    _connection,
    file_path: str = None,
    url: str = None,
    media_type: Optional[str] = None,
    name: Optional[str] = None,
    description: Optional[str] = None,
    callback_url: Optional[str] = None,
) -> dict:
    if not file_path and not url:
        raise VideodbError("Either file_path or url is required")
    if file_path and url:
        raise VideodbError("Only one of file_path or url is allowed")

    if file_path:
        try:
            name = file_path.split("/")[-1].split(".")[0] if not name else name
            upload_url_data = _connection.get(
                path=f"{ApiPath.collection}/{_connection.collection_id}/{ApiPath.upload_url}",
                params={"name": name},
            )
            upload_url = upload_url_data.get("upload_url")
            with open(file_path, "rb") as file:
                files = {"file": (name, file)}
                response = requests.post(upload_url, files=files)
                response.raise_for_status()
                url = upload_url

        except FileNotFoundError as e:
            raise VideodbError("File not found", cause=e)

        except HTTPError as e:
            raise VideodbError("Error while uploading file", cause=e)

    upload_data = _connection.post(
        path=f"{ApiPath.collection}/{_connection.collection_id}/{ApiPath.upload}",
        data={
            "url": url,
            "name": name,
            "description": description,
            "callback_url": callback_url,
            "media_type": media_type,
        },
    )
    return upload_data



from videodb._constants import (
    ApiPath,
)


class Audio:
    def __init__(self, _connection, id: str, collection_id: str, **kwargs) -> None:
        self._connection = _connection
        self.id = id
        self.collection_id = collection_id
        self.name = kwargs.get("name", None)
        self.length = kwargs.get("length", None)

    def __repr__(self) -> str:
        return (
            f"Audio("
            f"id={self.id}, "
            f"collection_id={self.collection_id}, "
            f"name={self.name}, "
            f"length={self.length})"
        )

    def delete(self) -> None:
        self._connection.delete(f"{ApiPath.audio}/{self.id}")



""" About information for videodb sdk"""


__version__ = "0.2.6"
__title__ = "videodb"
__author__ = "videodb"
__email__ = "contact@videodb.io"
__url__ = "https://github.com/video-db/videodb-python"
__license__ = "Apache License 2.0"



import copy
import logging
import uuid

from typing import Optional, Union

from videodb._constants import MaxSupported, TextStyle

logger = logging.getLogger(__name__)


def validate_max_supported(
    duration: Union[int, float], max_duration: Union[int, float], attribute: str = ""
) -> Union[int, float, None]:
    if duration is None:
        return 0
    if duration is not None and max_duration is not None and duration > max_duration:
        logger.warning(
            f"{attribute}: {duration} is greater than max supported: {max_duration}"
        )
    return duration


class MediaAsset:
    def __init__(self, asset_id: str) -> None:
        self.asset_id: str = asset_id

    def to_json(self) -> dict:
        return self.__dict__


class VideoAsset(MediaAsset):
    def __init__(
        self,
        asset_id: str,
        start: Optional[float] = 0,
        end: Optional[float] = None,
    ) -> None:
        super().__init__(asset_id)
        self.start: int = start
        self.end: Union[int, None] = end

    def to_json(self) -> dict:
        return copy.deepcopy(self.__dict__)

    def __repr__(self) -> str:
        return (
            f"VideoAsset("
            f"asset_id={self.asset_id}, "
            f"start={self.start}, "
            f"end={self.end})"
        )


class AudioAsset(MediaAsset):
    def __init__(
        self,
        asset_id: str,
        start: Optional[float] = 0,
        end: Optional[float] = None,
        disable_other_tracks: Optional[bool] = True,
        fade_in_duration: Optional[Union[int, float]] = 0,
        fade_out_duration: Optional[Union[int, float]] = 0,
    ):
        super().__init__(asset_id)
        self.start: int = start
        self.end: Union[int, None] = end
        self.disable_other_tracks: bool = disable_other_tracks
        self.fade_in_duration: Union[int, float] = validate_max_supported(
            fade_in_duration, MaxSupported.fade_duration, "fade_in_duration"
        )
        self.fade_out_duration: Union[int, float] = validate_max_supported(
            fade_out_duration, MaxSupported.fade_duration, "fade_out_duration"
        )

    def to_json(self) -> dict:
        return copy.deepcopy(self.__dict__)

    def __repr__(self) -> str:
        return (
            f"AudioAsset("
            f"asset_id={self.asset_id}, "
            f"start={self.start}, "
            f"end={self.end}, "
            f"disable_other_tracks={self.disable_other_tracks}, "
            f"fade_in_duration={self.fade_in_duration}, "
            f"fade_out_duration={self.fade_out_duration})"
        )


class ImageAsset(MediaAsset):
    def __init__(
        self,
        asset_id: str,
        width: Union[int, str] = 100,
        height: Union[int, str] = 100,
        x: Union[int, str] = 80,
        y: Union[int, str] = 20,
        duration: Optional[int] = None,
    ) -> None:
        super().__init__(asset_id)
        self.width = width
        self.height = height
        self.x = x
        self.y = y
        self.duration = duration

    def to_json(self) -> dict:
        return copy.deepcopy(self.__dict__)

    def __repr__(self) -> str:
        return (
            f"ImageAsset("
            f"asset_id={self.asset_id}, "
            f"width={self.width}, "
            f"height={self.height}, "
            f"x={self.x}, "
            f"y={self.y}, "
            f"duration={self.duration})"
        )


class TextAsset(MediaAsset):
    def __init__(
        self,
        text: str,
        duration: Optional[int] = None,
        style: TextStyle = TextStyle(),
    ) -> None:
        super().__init__(f"txt-{str(uuid.uuid4())}")
        self.text = text
        self.duration = duration
        self.style: TextStyle = style

    def to_json(self) -> dict:
        return {
            "text": copy.deepcopy(self.text),
            "asset_id": copy.deepcopy(self.asset_id),
            "duration": copy.deepcopy(self.duration),
            "style": copy.deepcopy(self.style.__dict__),
        }

    def __repr__(self) -> str:
        return (
            f"TextAsset("
            f"text={self.text}, "
            f"asset_id={self.asset_id}, "
            f"duration={self.duration}, "
            f"style={self.style})"
        )



from typing import List

from videodb._constants import ApiPath

from videodb.image import Frame


class Scene:
    def __init__(
        self,
        video_id: str,
        start: float,
        end: float,
        description: str,
        id: str = None,
        frames: List[Frame] = [],
        connection=None,
    ):
        self.id = id
        self.video_id = video_id
        self.start = start
        self.end = end
        self.frames: List[Frame] = frames
        self.description = description
        self._connection = connection

    def __repr__(self) -> str:
        return (
            f"Scene("
            f"id={self.id}, "
            f"video_id={self.video_id}, "
            f"start={self.start}, "
            f"end={self.end}, "
            f"frames={self.frames}, "
            f"description={self.description})"
        )

    def to_json(self):
        return {
            "id": self.id,
            "video_id": self.video_id,
            "start": self.start,
            "end": self.end,
            "frames": [frame.to_json() for frame in self.frames],
            "description": self.description,
        }

    def describe(self, prompt: str = None, model_name=None) -> None:
        if self._connection is None:
            raise ValueError("Connection is required to describe a scene")
        description_data = self._connection.post(
            path=f"{ApiPath.video}/{self.video_id}/{ApiPath.scene}/{self.id}/{ApiPath.describe}",
            data={"prompt": prompt, "model_name": model_name},
        )
        self.description = description_data.get("description", None)
        return self.description


class SceneCollection:
    def __init__(
        self,
        _connection,
        id: str,
        video_id: str,
        config: dict,
        scenes: List[Scene],
    ) -> None:
        self._connection = _connection
        self.id = id
        self.video_id = video_id
        self.config: dict = config
        self.scenes: List[Scene] = scenes

    def __repr__(self) -> str:
        return (
            f"SceneCollection("
            f"id={self.id}, "
            f"video_id={self.video_id}, "
            f"config={self.config}, "
            f"scenes={self.scenes})"
        )

    def delete(self) -> None:
        self._connection.delete(
            path=f"{ApiPath.video}/{self.video_id}/{ApiPath.scenes}/{self.id}"
        )



"""Videodb API client library"""

import os
import logging

from typing import Optional
from videodb._utils._video import play_stream
from videodb._constants import (
    VIDEO_DB_API,
    IndexType,
    SceneExtractionType,
    MediaType,
    SearchType,
    Segmenter,
    SubtitleAlignment,
    SubtitleBorderStyle,
    SubtitleStyle,
    TextStyle,
)
from videodb.client import Connection
from videodb.exceptions import (
    VideodbError,
    AuthenticationError,
    InvalidRequestError,
    SearchError,
)

logger: logging.Logger = logging.getLogger("videodb")


__all__ = [
    "VideodbError",
    "AuthenticationError",
    "InvalidRequestError",
    "IndexType",
    "SearchError",
    "play_stream",
    "MediaType",
    "SearchType",
    "SubtitleAlignment",
    "SubtitleBorderStyle",
    "SubtitleStyle",
    "TextStyle",
    "SceneExtractionType",
    "Segmenter",
]


def connect(
    api_key: str = None,
    base_url: Optional[str] = VIDEO_DB_API,
    log_level: Optional[int] = logging.INFO,
) -> Connection:
    """A client for interacting with a videodb via REST API

    :param str api_key: The api key to use for authentication
    :param str base_url: (optional) The base url to use for the api
    :param int log_level: (optional) The log level to use for the logger
    :return: A connection object
    :rtype: videodb.client.Connection
    """

    logger.setLevel(log_level)
    if api_key is None:
        api_key = os.environ.get("VIDEO_DB_API_KEY")
    if api_key is None:
        raise AuthenticationError(
            "No API key provided. Set an API key either as an environment variable (VIDEO_DB_API_KEY) or pass it as an argument."
        )

    return Connection(api_key, base_url)



"""This module contains the shot class"""


from typing import Optional
from videodb._utils._video import play_stream
from videodb._constants import (
    ApiPath,
)


class Shot:
    """A shot is a part of a video that contains a specific scene"""

    def __init__(
        self,
        _connection,
        video_id: str,
        video_length: float,
        video_title: str,
        start: float,
        end: float,
        text: Optional[str] = None,
        search_score: Optional[int] = None,
    ) -> None:
        self._connection = _connection
        self.video_id = video_id
        self.video_length = video_length
        self.video_title = video_title
        self.start = start
        self.end = end
        self.text = text
        self.search_score = search_score
        self.stream_url = None
        self.player_url = None

    def __repr__(self) -> str:
        return (
            f"Shot("
            f"video_id={self.video_id}, "
            f"video_title={self.video_title}, "
            f"start={self.start}, "
            f"end={self.end}, "
            f"text={self.text}, "
            f"search_score={self.search_score}, "
            f"stream_url={self.stream_url}, "
            f"player_url={self.player_url})"
        )

    def __getitem__(self, key):
        """Get an item from the shot object"""
        return self.__dict__[key]

    def generate_stream(self) -> str:
        """Generate a stream url for the shot

        :return: The stream url
        :rtype: str
        """

        if self.stream_url:
            return self.stream_url
        else:
            stream_data = self._connection.post(
                path=f"{ApiPath.video}/{self.video_id}/{ApiPath.stream}",
                data={
                    "timeline": [(self.start, self.end)],
                    "length": self.video_length,
                },
            )
            self.stream_url = stream_data.get("stream_url")
            self.player_url = stream_data.get("player_url")
            return self.stream_url

    def play(self) -> str:
        """Generate a stream url for the shot and open it in the default browser/ notebook

        :return: The stream url
        :rtype: str
        """
        self.generate_stream()
        return play_stream(self.stream_url)



from typing import Union

from videodb._constants import ApiPath
from videodb.asset import VideoAsset, AudioAsset, ImageAsset, TextAsset


class Timeline(object):
    def __init__(self, connection) -> None:
        self._connection = connection
        self._timeline = []
        self.stream_url = None
        self.player_url = None

    def to_json(self) -> dict:
        timeline_json = []
        for asset in self._timeline:
            if isinstance(asset, tuple):
                overlay_start, audio_asset = asset
                asset_json = audio_asset.to_json()
                asset_json["overlay_start"] = overlay_start
                timeline_json.append(asset_json)
            else:
                timeline_json.append(asset.to_json())
        return {"timeline": timeline_json}

    def add_inline(self, asset: VideoAsset) -> None:
        if not isinstance(asset, VideoAsset):
            raise ValueError("asset must be of type VideoAsset")
        self._timeline.append(asset)

    def add_overlay(
        self, start: int, asset: Union[AudioAsset, ImageAsset, TextAsset]
    ) -> None:
        if (
            not isinstance(asset, AudioAsset)
            and not isinstance(asset, ImageAsset)
            and not isinstance(asset, TextAsset)
        ):
            raise ValueError(
                "asset must be of type AudioAsset, ImageAsset or TextAsset"
            )
        self._timeline.append((start, asset))

    def generate_stream(self) -> str:
        stream_data = self._connection.post(
            path=f"{ApiPath.timeline}",
            data={
                "request_type": "compile",
                "timeline": self.to_json().get("timeline"),
            },
        )
        self.stream_url = stream_data.get("stream_url")
        self.player_url = stream_data.get("player_url")
        return stream_data.get("stream_url", None)



from typing import Optional, Union, List, Dict, Tuple
from videodb._utils._video import play_stream
from videodb._constants import (
    ApiPath,
    IndexType,
    SceneExtractionType,
    SearchType,
    Segmenter,
    SubtitleStyle,
    Workflows,
)
from videodb.image import Image, Frame
from videodb.scene import Scene, SceneCollection
from videodb.search import SearchFactory, SearchResult
from videodb.shot import Shot


class Video:
    def __init__(self, _connection, id: str, collection_id: str, **kwargs) -> None:
        self._connection = _connection
        self.id = id
        self.collection_id = collection_id
        self.stream_url = kwargs.get("stream_url", None)
        self.player_url = kwargs.get("player_url", None)
        self.name = kwargs.get("name", None)
        self.description = kwargs.get("description", None)
        self.thumbnail_url = kwargs.get("thumbnail_url", None)
        self.length = float(kwargs.get("length", 0.0))
        self.transcript = kwargs.get("transcript", None)
        self.transcript_text = kwargs.get("transcript_text", None)
        self.scenes = kwargs.get("scenes", None)

    def __repr__(self) -> str:
        return (
            f"Video("
            f"id={self.id}, "
            f"collection_id={self.collection_id}, "
            f"stream_url={self.stream_url}, "
            f"player_url={self.player_url}, "
            f"name={self.name}, "
            f"description={self.description}, "
            f"thumbnail_url={self.thumbnail_url}, "
            f"length={self.length})"
        )

    def __getitem__(self, key):
        return self.__dict__[key]

    def search(
        self,
        query: str,
        search_type: Optional[str] = SearchType.semantic,
        index_type: Optional[str] = IndexType.spoken_word,
        result_threshold: Optional[int] = None,
        score_threshold: Optional[float] = None,
        dynamic_score_percentage: Optional[float] = None,
        **kwargs,
    ) -> SearchResult:
        search = SearchFactory(self._connection).get_search(search_type)
        return search.search_inside_video(
            video_id=self.id,
            query=query,
            search_type=search_type,
            index_type=index_type,
            result_threshold=result_threshold,
            score_threshold=score_threshold,
            dynamic_score_percentage=dynamic_score_percentage,
            **kwargs,
        )

    def delete(self) -> None:
        """Delete the video

        :raises InvalidRequestError: If the delete fails
        :return: None if the delete is successful
        :rtype: None
        """
        self._connection.delete(path=f"{ApiPath.video}/{self.id}")

    def remove_storage(self) -> None:
        self._connection.delete(path=f"{ApiPath.video}/{self.id}/{ApiPath.storage}")

    def generate_stream(self, timeline: Optional[List[Tuple[int, int]]] = None) -> str:
        """Generate the stream url of the video

        :param list timeline: The timeline of the video to be streamed. Defaults to None.
        :raises InvalidRequestError: If the get_stream fails
        :return: The stream url of the video
        :rtype: str
        """
        if not timeline and self.stream_url:
            return self.stream_url

        stream_data = self._connection.post(
            path=f"{ApiPath.video}/{self.id}/{ApiPath.stream}",
            data={
                "timeline": timeline,
                "length": self.length,
            },
        )
        return stream_data.get("stream_url", None)

    def generate_thumbnail(self, time: Optional[float] = None) -> Union[str, Image]:
        if self.thumbnail_url and not time:
            return self.thumbnail_url

        if time:
            thumbnail_data = self._connection.post(
                path=f"{ApiPath.video}/{self.id}/{ApiPath.thumbnail}",
                data={
                    "time": time,
                },
            )
            return Image(self._connection, **thumbnail_data)

        thumbnail_data = self._connection.get(
            path=f"{ApiPath.video}/{self.id}/{ApiPath.thumbnail}"
        )
        self.thumbnail_url = thumbnail_data.get("thumbnail_url")
        return self.thumbnail_url

    def get_thumbnails(self) -> List[Image]:
        thumbnails_data = self._connection.get(
            path=f"{ApiPath.video}/{self.id}/{ApiPath.thumbnails}"
        )
        return [Image(self._connection, **thumbnail) for thumbnail in thumbnails_data]

    def _fetch_transcript(
        self,
        start: int = None,
        end: int = None,
        segmenter: str = Segmenter.word,
        length: int = 1,
        force: bool = None,
    ) -> None:
        if (
            self.transcript
            and not start
            and not end
            and not segmenter
            and not length
            and not force
        ):
            return
        transcript_data = self._connection.get(
            path=f"{ApiPath.video}/{self.id}/{ApiPath.transcription}",
            params={
                "start": start,
                "end": end,
                "segmenter": segmenter,
                "length": length,
                "force": "true" if force else "false",
            },
            show_progress=True,
        )
        self.transcript = transcript_data.get("word_timestamps", [])
        self.transcript_text = transcript_data.get("text", "")

    def get_transcript(
        self,
        start: int = None,
        end: int = None,
        segmenter: str = Segmenter.word,
        length: int = 1,
        force: bool = None,
    ) -> List[Dict]:
        self._fetch_transcript(
            start=start, end=end, segmenter=segmenter, length=length, force=force
        )
        return self.transcript

    def get_transcript_text(
        self,
        start: int = None,
        end: int = None,
        segmenter: str = Segmenter.word,
        length: int = 1,
        force: bool = None,
    ) -> str:
        self._fetch_transcript(
            start=start, end=end, segmenter=segmenter, length=length, force=force
        )
        return self.transcript_text

    def index_spoken_words(
        self,
        language_code: Optional[str] = None,
        force: bool = False,
        callback_url: str = None,
    ) -> None:
        """Semantic indexing of spoken words in the video

        :raises InvalidRequestError: If the video is already indexed
        :return: None if the indexing is successful
        :rtype: None
        """
        self._connection.post(
            path=f"{ApiPath.video}/{self.id}/{ApiPath.index}",
            data={
                "index_type": IndexType.spoken_word,
                "language_code": language_code,
                "force": force,
                "callback_url": callback_url,
            },
            show_progress=True,
        )

    def get_scenes(self) -> Union[list, None]:
        if self.scenes:
            return self.scenes
        scene_data = self._connection.get(
            path=f"{ApiPath.video}/{self.id}/{ApiPath.index}",
            params={
                "index_type": IndexType.scene,
            },
        )
        self.scenes = scene_data
        return scene_data if scene_data else None

    def _format_scene_collection(self, scene_collection_data: dict) -> SceneCollection:
        scenes = []
        for scene in scene_collection_data.get("scenes", []):
            frames = []
            for frame in scene.get("frames", []):
                frame = Frame(
                    self._connection,
                    frame.get("frame_id"),
                    self.id,
                    scene.get("scene_id"),
                    frame.get("url"),
                    frame.get("frame_time"),
                    frame.get("description"),
                )
                frames.append(frame)
            scene = Scene(
                video_id=self.id,
                start=scene.get("start"),
                end=scene.get("end"),
                description=scene.get("description"),
                id=scene.get("scene_id"),
                frames=frames,
                connection=self._connection,
            )
            scenes.append(scene)

        return SceneCollection(
            self._connection,
            scene_collection_data.get("scene_collection_id"),
            self.id,
            scene_collection_data.get("config", {}),
            scenes,
        )

    def extract_scenes(
        self,
        extraction_type: SceneExtractionType = SceneExtractionType.shot_based,
        extraction_config: dict = {},
        force: bool = False,
        callback_url: str = None,
    ) -> Optional[SceneCollection]:
        scenes_data = self._connection.post(
            path=f"{ApiPath.video}/{self.id}/{ApiPath.scenes}",
            data={
                "extraction_type": extraction_type,
                "extraction_config": extraction_config,
                "force": force,
                "callback_url": callback_url,
            },
        )
        if not scenes_data:
            return None
        return self._format_scene_collection(scenes_data.get("scene_collection"))

    def get_scene_collection(self, collection_id: str) -> Optional[SceneCollection]:
        if not collection_id:
            raise ValueError("collection_id is required")
        scenes_data = self._connection.get(
            path=f"{ApiPath.video}/{self.id}/{ApiPath.scenes}/{collection_id}"
        )
        if not scenes_data:
            return None
        return self._format_scene_collection(scenes_data.get("scene_collection"))

    def list_scene_collection(self):
        scene_collections_data = self._connection.get(
            path=f"{ApiPath.video}/{self.id}/{ApiPath.scenes}"
        )
        return scene_collections_data.get("scene_collections", [])

    def delete_scene_collection(self, collection_id: str) -> None:
        if not collection_id:
            raise ValueError("collection_id is required")
        self._connection.delete(
            path=f"{ApiPath.video}/{self.id}/{ApiPath.scenes}/{collection_id}"
        )

    def index_scenes(
        self,
        extraction_type: SceneExtractionType = SceneExtractionType.shot_based,
        extraction_config: Dict = {},
        prompt: Optional[str] = None,
        model_name: Optional[str] = None,
        model_config: Optional[Dict] = None,
        name: Optional[str] = None,
        scenes: Optional[List[Scene]] = None,
        callback_url: Optional[str] = None,
    ) -> Optional[str]:
        scenes_data = self._connection.post(
            path=f"{ApiPath.video}/{self.id}/{ApiPath.index}/{ApiPath.scene}",
            data={
                "extraction_type": extraction_type,
                "extraction_config": extraction_config,
                "prompt": prompt,
                "model_name": model_name,
                "model_config": model_config,
                "name": name,
                "scenes": [scene.to_json() for scene in scenes] if scenes else None,
                "callback_url": callback_url,
            },
        )
        if not scenes_data:
            return None
        return scenes_data.get("scene_index_id")

    def list_scene_index(self) -> List:
        index_data = self._connection.get(
            path=f"{ApiPath.video}/{self.id}/{ApiPath.index}/{ApiPath.scene}"
        )
        return index_data.get("scene_indexes", [])

    def get_scene_index(self, scene_index_id: str) -> Optional[List]:
        index_data = self._connection.get(
            path=f"{ApiPath.video}/{self.id}/{ApiPath.index}/{ApiPath.scene}/{scene_index_id}"
        )
        if not index_data:
            return None
        return index_data.get("scene_index_records", [])

    def delete_scene_index(self, scene_index_id: str) -> None:
        if not scene_index_id:
            raise ValueError("scene_index_id is required")
        self._connection.delete(
            path=f"{ApiPath.video}/{self.id}/{ApiPath.index}/{ApiPath.scene}/{scene_index_id}"
        )

    def add_subtitle(self, style: SubtitleStyle = SubtitleStyle()) -> str:
        if not isinstance(style, SubtitleStyle):
            raise ValueError("style must be of type SubtitleStyle")
        subtitle_data = self._connection.post(
            path=f"{ApiPath.video}/{self.id}/{ApiPath.workflow}",
            data={
                "type": Workflows.add_subtitles,
                "subtitle_style": style.__dict__,
            },
        )
        return subtitle_data.get("stream_url", None)

    def insert_video(self, video, timestamp: float) -> str:
        """Insert a video into another video

        :param Video video: The video to be inserted
        :param float timestamp: The timestamp where the video should be inserted
        :raises InvalidRequestError: If the insert fails
        :return: The stream url of the inserted video
        :rtype: str
        """
        if timestamp > float(self.length):
            timestamp = float(self.length)

        pre_shot = Shot(self._connection, self.id, timestamp, "", 0, timestamp)
        inserted_shot = Shot(
            self._connection, video.id, video.length, "", 0, video.length
        )
        post_shot = Shot(
            self._connection,
            self.id,
            self.length - timestamp,
            "",
            timestamp,
            self.length,
        )
        all_shots = [pre_shot, inserted_shot, post_shot]

        compile_data = self._connection.post(
            path=f"{ApiPath.compile}",
            data=[
                {
                    "video_id": shot.video_id,
                    "collection_id": self.collection_id,
                    "shots": [(float(shot.start), float(shot.end))],
                }
                for shot in all_shots
            ],
        )
        return compile_data.get("stream_url", None)

    def play(self) -> str:
        """Open the player url in the browser/iframe and return the stream url

        :return: The stream url
        :rtype: str
        """
        return play_stream(self.stream_url)



import logging

from typing import (
    Optional,
    Union,
    List,
)
from videodb.__about__ import __version__
from videodb._constants import (
    ApiPath,
)

from videodb.collection import Collection
from videodb._utils._http_client import HttpClient
from videodb.video import Video
from videodb.audio import Audio
from videodb.image import Image

from videodb._upload import (
    upload,
)

logger = logging.getLogger(__name__)


class Connection(HttpClient):
    def __init__(self, api_key: str, base_url: str) -> None:
        self.api_key = api_key
        self.base_url = base_url
        self.collection_id = "default"
        super().__init__(api_key=api_key, base_url=base_url, version=__version__)

    def get_collection(self, collection_id: Optional[str] = "default") -> Collection:
        collection_data = self.get(path=f"{ApiPath.collection}/{collection_id}")
        self.collection_id = collection_data.get("id", "default")
        return Collection(
            self,
            self.collection_id,
            collection_data.get("name"),
            collection_data.get("description"),
        )

    def get_collections(self) -> List[Collection]:
        collections_data = self.get(path=ApiPath.collection)
        return [
            Collection(
                self,
                collection.get("id"),
                collection.get("name"),
                collection.get("description"),
            )
            for collection in collections_data.get("collections")
        ]

    def create_collection(self, name: str, description: str) -> Collection:
        collection_data = self.post(
            path=ApiPath.collection,
            data={
                "name": name,
                "description": description,
            },
        )
        self.collection_id = collection_data.get("id", "default")
        return Collection(
            self,
            collection_data.get("id"),
            collection_data.get("name"),
            collection_data.get("description"),
        )

    def update_collection(self, id: str, name: str, description: str) -> Collection:
        collection_data = self.patch(
            path=f"{ApiPath.collection}/{id}",
            data={
                "name": name,
                "description": description,
            },
        )
        self.collection_id = collection_data.get("id", "default")
        return Collection(
            self,
            collection_data.get("id"),
            collection_data.get("name"),
            collection_data.get("description"),
        )

    def check_usage(self) -> dict:
        return self.get(path=f"{ApiPath.billing}/{ApiPath.usage}")

    def get_invoices(self) -> List[dict]:
        return self.get(path=f"{ApiPath.billing}/{ApiPath.invoices}")

    def download(self, stream_link: str, name: str) -> dict:
        return self.post(
            path=f"{ApiPath.download}",
            data={
                "stream_link": stream_link,
                "name": name,
            },
        )

    def upload(
        self,
        file_path: str = None,
        url: str = None,
        media_type: Optional[str] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
        callback_url: Optional[str] = None,
    ) -> Union[Video, Audio, Image, None]:
        upload_data = upload(
            self,
            file_path,
            url,
            media_type,
            name,
            description,
            callback_url,
        )
        media_id = upload_data.get("id", "")
        if media_id.startswith("m-"):
            return Video(self, **upload_data)
        elif media_id.startswith("a-"):
            return Audio(self, **upload_data)
        elif media_id.startswith("img-"):
            return Image(self, **upload_data)



from abc import ABC, abstractmethod
from videodb._utils._video import play_stream
from videodb._constants import (
    IndexType,
    SearchType,
    ApiPath,
    SemanticSearchDefaultValues,
)
from videodb.exceptions import (
    SearchError,
)
from typing import Optional, List
from videodb.shot import Shot


class SearchResult:
    def __init__(self, _connection, **kwargs):
        self._connection = _connection
        self.shots = []
        self.stream_url = None
        self.player_url = None
        self.collection_id = "default"
        self._results = kwargs.get("results", [])
        self._format_results()

    def _format_results(self):
        for result in self._results:
            self.collection_id = result.get("collection_id")
            for doc in result.get("docs"):
                self.shots.append(
                    Shot(
                        self._connection,
                        result.get("video_id"),
                        result.get("length"),
                        result.get("title"),
                        doc.get("start"),
                        doc.get("end"),
                        doc.get("text"),
                        doc.get("score"),
                    )
                )

    def __repr__(self) -> str:
        return (
            f"SearchResult("
            f"collection_id={self.collection_id}, "
            f"stream_url={self.stream_url}, "
            f"player_url={self.player_url}, "
            f"shots={self.shots})"
        )

    def get_shots(self) -> List[Shot]:
        return self.shots

    def compile(self) -> str:
        """Compile the search result shots into a stream url

        :raises SearchError: If no shots are found in the search results
        :return: The stream url
        :rtype: str
        """
        if self.stream_url:
            return self.stream_url
        elif self.shots:
            compile_data = self._connection.post(
                path=f"{ApiPath.compile}",
                data=[
                    {
                        "video_id": shot.video_id,
                        "collection_id": self.collection_id,
                        "shots": [(shot.start, shot.end)],
                    }
                    for shot in self.shots
                ],
            )
            self.stream_url = compile_data.get("stream_url")
            self.player_url = compile_data.get("player_url")
            return self.stream_url

        else:
            raise SearchError("No shots found in search results to compile")

    def play(self) -> str:
        """Generate a stream url for the shot and open it in the default browser

        :return: The stream url
        :rtype: str
        """
        self.compile()
        return play_stream(self.stream_url)


class Search(ABC):
    """Search interface inside video or collection"""

    @abstractmethod
    def search_inside_video(self, *args, **kwargs):
        pass

    @abstractmethod
    def search_inside_collection(self, *args, **kwargs):
        pass


class SemanticSearch(Search):
    def __init__(self, _connection):
        self._connection = _connection

    def search_inside_video(
        self,
        video_id: str,
        query: str,
        search_type: str,
        index_type: str,
        result_threshold: Optional[int] = None,
        score_threshold: Optional[float] = None,
        dynamic_score_percentage: Optional[float] = None,
        **kwargs,
    ):
        search_data = self._connection.post(
            path=f"{ApiPath.video}/{video_id}/{ApiPath.search}",
            data={
                "search_type": search_type,
                "index_type": index_type,
                "query": query,
                "score_threshold": score_threshold
                or SemanticSearchDefaultValues.score_threshold,
                "result_threshold": result_threshold
                or SemanticSearchDefaultValues.result_threshold,
                "dynamic_score_percentage": dynamic_score_percentage,
                **kwargs,
            },
        )
        return SearchResult(self._connection, **search_data)

    def search_inside_collection(
        self,
        collection_id: str,
        query: str,
        search_type: str,
        index_type: str,
        result_threshold: Optional[int] = None,
        score_threshold: Optional[float] = None,
        dynamic_score_percentage: Optional[float] = None,
        **kwargs,
    ):
        search_data = self._connection.post(
            path=f"{ApiPath.collection}/{collection_id}/{ApiPath.search}",
            data={
                "search_type": search_type,
                "index_type": index_type,
                "query": query,
                "score_threshold": score_threshold
                or SemanticSearchDefaultValues.score_threshold,
                "result_threshold": result_threshold
                or SemanticSearchDefaultValues.result_threshold,
                "dynamic_score_percentage": dynamic_score_percentage,
                **kwargs,
            },
        )
        return SearchResult(self._connection, **search_data)


class KeywordSearch(Search):
    def __init__(self, _connection):
        self._connection = _connection

    def search_inside_video(
        self,
        video_id: str,
        query: str,
        search_type: str,
        index_type: str,
        result_threshold: Optional[int] = None,
        score_threshold: Optional[float] = None,
        dynamic_score_percentage: Optional[float] = None,
        **kwargs,
    ):
        search_data = self._connection.post(
            path=f"{ApiPath.video}/{video_id}/{ApiPath.search}",
            data={
                "search_type": search_type,
                "index_type": index_type,
                "query": query,
                "score_threshold": score_threshold,
                "result_threshold": result_threshold,
                **kwargs,
            },
        )
        return SearchResult(self._connection, **search_data)

    def search_inside_collection(self, **kwargs):
        raise NotImplementedError("Keyword search will be implemented in the future")


class SceneSearch(Search):
    def __init__(self, _connection):
        self._connection = _connection

    def search_inside_video(
        self,
        video_id: str,
        query: str,
        search_type: str,
        index_type: str,
        result_threshold: Optional[int] = None,
        score_threshold: Optional[float] = None,
        dynamic_score_percentage: Optional[float] = None,
        **kwargs,
    ):
        search_data = self._connection.post(
            path=f"{ApiPath.video}/{video_id}/{ApiPath.search}",
            data={
                "search_type": search_type,
                "index_type": IndexType.scene,
                "query": query,
                "score_threshold": score_threshold,
                "result_threshold": result_threshold,
                "dynamic_score_percentage": dynamic_score_percentage,
                **kwargs,
            },
        )
        return SearchResult(self._connection, **search_data)

    def search_inside_collection(self, **kwargs):
        raise NotImplementedError("Scene search will be implemented in the future")


search_type = {
    SearchType.semantic: SemanticSearch,
    SearchType.keyword: KeywordSearch,
    SearchType.scene: SceneSearch,
}


class SearchFactory:
    def __init__(self, _connection):
        self._connection = _connection

    def get_search(self, type: str):
        if type not in search_type:
            raise SearchError(
                f"Invalid search type: {type}. Valid search types are: {list(search_type.keys())}"
            )
        return search_type[type](self._connection)



# package setup
import os
from setuptools import setup, find_packages

ROOT = os.path.dirname(os.path.abspath(__file__))


# Read in the package version per recommendations from:
# https://packaging.python.org/guides/single-sourcing-package-version/

about_path = os.path.join(ROOT, "videodb", "__about__.py")
about = {}
with open(about_path) as fp:
    exec(fp.read(), about)


# read the contents of README file
long_description = open(os.path.join(ROOT, "README.md"), "r", encoding="utf-8").read()


setup(
    name=about["__title__"],
    version=about["__version__"],
    author=about["__author__"],
    author_email=about["__email__"],
    license=about["__license__"],
    description="VideoDB Python SDK",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=about["__url__"],
    packages=find_packages(exclude=["tests", "tests.*"]),
    python_requires=">=3.8",
    install_requires=[
        "requests>=2.25.1",
        "backoff>=2.2.1",
        "tqdm>=4.66.1",
    ],
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "License :: OSI Approved :: Apache Software License",
    ],
)



import logging

from typing import (
    Optional,
    Union,
    List,
)
from videodb._upload import (
    upload,
)
from videodb._constants import (
    ApiPath,
    IndexType,
    SearchType,
)
from videodb.video import Video
from videodb.audio import Audio
from videodb.image import Image
from videodb.search import SearchFactory, SearchResult

logger = logging.getLogger(__name__)


class Collection:
    def __init__(self, _connection, id: str, name: str = None, description: str = None):
        self._connection = _connection
        self.id = id
        self.name = name
        self.description = description

    def __repr__(self) -> str:
        return (
            f"Collection("
            f"id={self.id}, "
            f"name={self.name}, "
            f"description={self.description})"
        )

    def get_videos(self) -> List[Video]:
        videos_data = self._connection.get(
            path=f"{ApiPath.video}",
            params={"collection_id": self.id},
        )
        return [Video(self._connection, **video) for video in videos_data.get("videos")]

    def get_video(self, video_id: str) -> Video:
        video_data = self._connection.get(
            path=f"{ApiPath.video}/{video_id}", params={"collection_id": self.id}
        )
        return Video(self._connection, **video_data)

    def delete_video(self, video_id: str) -> None:
        """Delete the video

        :param str video_id: The id of the video to be deleted
        :raises InvalidRequestError: If the delete fails
        :return: None if the delete is successful
        :rtype: None
        """
        return self._connection.delete(
            path=f"{ApiPath.video}/{video_id}", params={"collection_id": self.id}
        )

    def get_audios(self) -> List[Audio]:
        audios_data = self._connection.get(
            path=f"{ApiPath.audio}",
            params={"collection_id": self.id},
        )
        return [Audio(self._connection, **audio) for audio in audios_data.get("audios")]

    def get_audio(self, audio_id: str) -> Audio:
        audio_data = self._connection.get(
            path=f"{ApiPath.audio}/{audio_id}", params={"collection_id": self.id}
        )
        return Audio(self._connection, **audio_data)

    def delete_audio(self, audio_id: str) -> None:
        return self._connection.delete(
            path=f"{ApiPath.audio}/{audio_id}", params={"collection_id": self.id}
        )

    def get_images(self) -> List[Image]:
        images_data = self._connection.get(
            path=f"{ApiPath.image}",
            params={"collection_id": self.id},
        )
        return [Image(self._connection, **image) for image in images_data.get("images")]

    def get_image(self, image_id: str) -> Image:
        image_data = self._connection.get(
            path=f"{ApiPath.image}/{image_id}", params={"collection_id": self.id}
        )
        return Image(self._connection, **image_data)

    def delete_image(self, image_id: str) -> None:
        return self._connection.delete(
            path=f"{ApiPath.image}/{image_id}", params={"collection_id": self.id}
        )

    def search(
        self,
        query: str,
        search_type: Optional[str] = SearchType.semantic,
        index_type: Optional[str] = IndexType.spoken_word,
        result_threshold: Optional[int] = None,
        score_threshold: Optional[float] = None,
        dynamic_score_percentage: Optional[float] = None,
    ) -> SearchResult:
        search = SearchFactory(self._connection).get_search(search_type)
        return search.search_inside_collection(
            collection_id=self.id,
            query=query,
            search_type=search_type,
            index_type=index_type,
            result_threshold=result_threshold,
            score_threshold=score_threshold,
            dynamic_score_percentage=dynamic_score_percentage,
        )

    def search_title(self, query) -> List[Video]:
        search_data = self._connection.post(
            path=f"{ApiPath.collection}/{self.id}/{ApiPath.search}/{ApiPath.title}",
            data={
                "query": query,
                "search_type": SearchType.llm,
            },
        )
        return [
            {"video": Video(self._connection, **result.get("video"))}
            for result in search_data
        ]

    def upload(
        self,
        file_path: str = None,
        url: Optional[str] = None,
        media_type: Optional[str] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
        callback_url: Optional[str] = None,
    ) -> Union[Video, Audio, Image, None]:
        upload_data = upload(
            self._connection,
            file_path,
            url,
            media_type,
            name,
            description,
            callback_url,
        )
        media_id = upload_data.get("id", "")
        if media_id.startswith("m-"):
            return Video(self._connection, **upload_data)
        elif media_id.startswith("a-"):
            return Audio(self._connection, **upload_data)
        elif media_id.startswith("img-"):
            return Image(self._connection, **upload_data)
